#
# spec file for package go1.14
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


# strip will cause Go's .a archives to become invalid because strip appears to
# reassemble the archive incorrectly. This is a known issue upstream
# (https://github.com/golang/go/issues/17890), but we have to deal with it in
# the meantime.
%undefine _build_create_debug
%define __arch_install_post export NO_BRP_STRIP_DEBUG=true NO_BRP_AR=true

%if 0%{?suse_version} == 1315
%define gcc_go_version 6
%define go_bootstrap_version go1.4
%else
%define go_bootstrap_version go1.9
%if 0%{?sle_version} == 150000
# SLE15 or Leap 15.x
%define gcc_go_version 7
%else
%define gcc_go_version 9
%endif
%endif

# By default use go and not gccgo
%bcond_with    gccgo

# The fallback boostrap method via %%{go_bootstrap_version} would work for Leap
# but we don't have %%{go_bootstrap_version} in there. Same for SLE15+
#if ( 0%{?suse_version} < 1550 && 0%{?is_opensuse} ) || ( 0%{?suse_version} >= 1500 && ! 0%{?is_opensuse} )
#bcond_without gccgo
#endif

# The fallback bootstrap method via go1.4 doesn't work
# for aarch64 nor ppc64le because go 1.4 did not support that architecture.
%if 0%{?suse_version} == 1315
%ifarch aarch64 ppc64le ppc64 s390x
%bcond_without gccgo
%endif
%endif

# gccgo on ppc64le with default PIE enabled fails with:
# error while loading shared libraries:
# R_PPC64_ADDR16_HA re10143fb0c for symbol `' out of range
# track https://github.com/golang/go/issues/28531
# linuxppc-dev discussion:
# "PIE binaries are no longer mapped below 4 GiB on ppc64le"
# https://lists.ozlabs.org/pipermail/linuxppc-dev/2018-November/180862.html
%ifarch ppc64le
#!BuildIgnore: gcc-PIE
%endif

# By default we don't include tsan. It's only supported on amd64.
%define tsan_arch x86_64

# Go has precompiled versions of LLVM's compiler-rt inside their source code.
# We cannot ship pre-compiled binaries so we have to recompile said source,
# however they vendor specific commits from upstream. This value comes from
# src/runtime/race/README (and we verify that it matches in check).
# See boo#1052528 for more details.
%define tsan_commit 810ae8ddac890a6613d814c0b5415c7fcb7f5cca

%define go_api 1.14

# shared library support
%if "%{rpm_vercmp %{go_api} 1.5}" > "0"
%if %{with gccgo}
%define with_shared 1
%else
%ifarch %ix86 %arm x86_64 aarch64
%define with_shared 1
%else
%define with_shared 0
%endif
%endif
%else
%define with_shared 0
%endif
%ifarch ppc64
%define with_shared 0
%endif
# setup go_arch (BSD-like scheme)
%ifarch %ix86
%define go_arch 386
%endif
%ifarch x86_64
%define go_arch amd64
%endif
%ifarch aarch64
%define go_arch arm64
%endif
%ifarch %arm
%define go_arch arm
%endif
%ifarch ppc64
%define go_arch ppc64
%endif
%ifarch ppc64le
%define go_arch ppc64le
%endif
%ifarch s390x
%define go_arch s390x
%endif

Name:           go1.14
Version:        1.14.2
Release:        lp151.10.1
Summary:        A compiled, garbage-collected, concurrent programming language
License:        BSD-3-Clause
Group:          Development/Languages/Other
URL:            http://golang.org
Source:         http://golang.org/dl/go%{version}.src.tar.gz
Source1:        go-rpmlintrc
Source4:        README.SUSE
Source6:        go.gdbinit
# We have to compile TSAN ourselves. boo#1052528
Source100:      compiler-rt-g%{tsan_commit}.tar.xz
# PATCH-FIX-OPENSUSE enable writing tools outside $GOROOT/pkg/tool for packaging
Patch5:         tools-packaging.patch
# PATCH-FIX-UPSTREAM marguerite@opensuse.org - find /usr/bin/go-5 when bootstrapping with gcc5-go
Patch8:         gcc6-go.patch
Patch9:         gcc7-go.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
# boostrap
%if %{with gccgo}
%ifnarch s390 s390x
BuildRequires:  binutils-gold
%endif
BuildRequires:  gcc%{gcc_go_version}-go
%else
# no gcc-go
BuildRequires:  %{go_bootstrap_version}
%endif
BuildRequires:  fdupes
Recommends:     %{name}-doc = %{version}
%ifarch %{tsan_arch}
# Needed to compile compiler-rt/TSAN.
BuildRequires:  gcc-c++
%endif
#BNC#818502 debug edit tool of rpm fails on i586 builds
BuildRequires:  rpm >= 4.11.1
# Needed on arm aarch64 to avoid
# collect2: fatal error: cannot find 'ld'-
%ifarch %arm aarch64
BuildRequires:  binutils-gold
%endif
Requires(post):	update-alternatives
Requires(postun):	update-alternatives
Requires:       gcc
Provides:       go = %{version}
Provides:       go-devel = go%{version}
Provides:       go-devel-static = go%{version}
Provides:       golang(API) = %{go_api}
Obsoletes:      go-devel < go%{version}
# go-vim/emacs were separate projects starting from 1.4
Obsoletes:      go-emacs <= 1.3.3
Obsoletes:      go-vim <= 1.3.3
ExclusiveArch:  %ix86 x86_64 %arm aarch64 ppc64 ppc64le s390x

# currently a bad html in it at 1.14.2
#!BuildIgnore:	post-build-checks-malwarescan

%description
Go is an expressive, concurrent, garbage collected systems programming language
that is type safe and memory safe. It has pointers but no pointer arithmetic.
Go has fast builds, clean syntax, garbage collection, methods for any type, and
run-time reflection. It feels like a dynamic language but has the speed and
safety of a static language.

%package doc
Summary:        Go documentation
Group:          Documentation/Other
Requires:       %{name} = %{version}
Provides:       go-doc = %{version}

%description doc
Go examples and documentation.

%ifarch %{tsan_arch}
# boo#1052528
%package race
Summary:        Go runtime race detector
Group:          Development/Languages/Other
URL:            https://compiler-rt.llvm.org/
Requires:       %{name} = %{version}
Supplements:    %{name} = %{version}
ExclusiveArch:  %{tsan_arch}

%description race
Go runtime race detector libraries. Install this package if you wish to use the
-race option, in order to detect race conditions present in your Go programs.
%endif

%prep
%ifarch %{tsan_arch}
# compiler-rt
%setup -q -T -b 100 -n compiler-rt-g%{tsan_commit}
%endif
# go
%setup -q -n go
%patch5 -p1
%if %{with gccgo}
%if 0%{?gcc_go_version} == 6
%patch8 -p1
%endif
%if 0%{?gcc_go_version} == 7
%patch9 -p1
%endif
%endif

cp %{SOURCE4} .

%build
# Remove the pre-included .sysos, to avoid shipping things we didn't compile
# (which is against the openSUSE guidelines for packaging).
find . -type f -name '*.syso' -print -delete

# First, compile LLVM's TSAN, and replace the built-in with it. We can only do
# this for amd64.
%ifarch %{tsan_arch}
pushd ../compiler-rt*/lib/tsan/go
./buildgo.sh
popd
cp ../compiler-rt*/lib/tsan/go/race_linux_%{go_arch}.syso src/runtime/race/race_linux_%{go_arch}.syso
%endif

# Now, compile Go.
%if %{with gccgo}
export GOROOT_BOOTSTRAP=%{_prefix}
%else
export GOROOT_BOOTSTRAP=%{_libdir}/%{go_bootstrap_version}
%endif
export GOROOT="`pwd`"
export GOROOT_FINAL=%{_libdir}/go/%{go_api}
export GOBIN="$GOROOT/bin"
mkdir -p "$GOBIN"
cd src
HOST_EXTRA_CFLAGS="%{optflags} -Wno-error" ./make.bash -v

cd ../
%ifarch %{tsan_arch}
# Install TSAN-friendly version of the std libraries.
bin/go install -race std
%endif

%if %{with_shared}
bin/go install -buildmode=shared -linkshared std
%endif

%check
%ifarch %{tsan_arch}
# Make sure that we have the right TSAN checked out.
grep "%{tsan_commit}" src/runtime/race/README
%endif

%install
export GOROOT="%{buildroot}%{_libdir}/go/%{go_api}"

# locations for third party libraries, see README-openSUSE for info about locations.
install -d  %{buildroot}%{_datadir}/go/%{go_api}/contrib
install -d  $GOROOT/contrib/pkg/linux_%{go_arch}
ln -s %{_libdir}/go/%{go_api}/contrib/pkg/ %{buildroot}%{_datadir}/go/%{go_api}/contrib/pkg
install -d  %{buildroot}%{_datadir}/go/%{go_api}/contrib/cmd
install -d  %{buildroot}%{_datadir}/go/%{go_api}/contrib/src
ln -s %{_datadir}/go/%{go_api}/contrib/src/ %{buildroot}%{_libdir}/go/%{go_api}/contrib/src
install -Dm644 README.SUSE $GOROOT/contrib/
ln -s %{_libdir}/go/%{go_api}/contrib/README.SUSE %{buildroot}%{_datadir}/go/%{go_api}/contrib/README.SUSE

# source files for go install, godoc, etc
install -d %{buildroot}%{_datadir}/go/%{go_api}
for ext in *.{go,c,h,s,S,py,syso}; do
  find src -name ${ext} -exec install -Dm644 \{\} %{buildroot}%{_datadir}/go/%{go_api}/\{\} \;
done
mkdir -p $GOROOT/src
for i in $(ls %{buildroot}/usr/share/go/%{go_api}/src);do
  ln -s /usr/share/go/%{go_api}/src/$i $GOROOT/src/$i
done
# add lib files that are needed (such as the timezone database).
install -d $GOROOT/lib
find lib -type f -exec install -D -m644 {} $GOROOT/{} \;

# copy document templates, packages, obj libs and command utilities
mkdir -p $GOROOT/bin
# remove bootstrap
rm -rf pkg/bootstrap
mv pkg $GOROOT
mv bin/* $GOROOT/bin
mkdir -p $GOROOT/misc/trace
mv misc/trace/* $GOROOT/misc/trace
# add wasm (Web Assembly) boo#1139210
mkdir -p $GOROOT/misc/wasm
mv misc/wasm/* $GOROOT/misc/wasm
rm -f %{buildroot}%{_bindir}/{hgpatch,quietgcc}

# gdbinit
install -Dm644 %{SOURCE6} $GOROOT/bin/gdbinit.d/go.gdb
%if "%{_lib}" == "lib64"
sed -i "s/lib/lib64/" $GOROOT/bin/gdbinit.d/go.gdb
sed -i "s/\$go_api/%{go_api}/" $GOROOT/bin/gdbinit.d/go.gdb
%endif

# update-alternatives
 mkdir -p %{buildroot}%{_sysconfdir}/alternatives
 mkdir -p %{buildroot}%{_bindir}
 mkdir -p %{buildroot}%{_sysconfdir}/profile.d
 mkdir -p %{buildroot}%{_sysconfdir}/gdbinit.d
 touch %{buildroot}%{_sysconfdir}/alternatives/{go,gofmt,go.gdb}
 ln -sf %{_sysconfdir}/alternatives/go %{buildroot}%{_bindir}/go
 ln -sf %{_sysconfdir}/alternatives/gofmt %{buildroot}%{_bindir}/gofmt
 ln -sf %{_sysconfdir}/alternatives/go.gdb %{buildroot}%{_sysconfdir}/gdbinit.d/go.gdb

# documentation and examples
# fix documetation permissions (rpmlint warning)
find doc/ misc/ -type f -exec chmod 0644 '{}' +
# remove unwanted arch-dependant binaries (rpmlint warning)
rm -rf misc/cgo/test/{_*,*.o,*.out,*.6,*.8}
# prepare go-doc
mkdir -p %{buildroot}%{_docdir}/go/%{go_api}
cp -r AUTHORS CONTRIBUTORS CONTRIBUTING.md LICENSE PATENTS README.md README.SUSE %{buildroot}%{_docdir}/go/%{go_api}
cp -r doc/* %{buildroot}%{_docdir}/go/%{go_api}

%fdupes -s %{buildroot}%{_prefix}

%post

update-alternatives \
  --install %{_bindir}/go go %{_libdir}/go/%{go_api}/bin/go 30 \
  --slave %{_bindir}/gofmt gofmt %{_libdir}/go/%{go_api}/bin/gofmt \
  --slave %{_sysconfdir}/gdbinit.d/go.gdb go.gdb %{_libdir}/go/%{go_api}/bin/gdbinit.d/go.gdb

%postun
if [ $1 -eq 0 ] ; then
	update-alternatives --remove go %{_libdir}/go/%{go_api}/bin/go
fi

%files
%defattr(-,root,root,-)
%{_bindir}/go
%{_bindir}/gofmt
%dir %{_libdir}/go
%{_libdir}/go/%{go_api}
%dir %{_datadir}/go
%{_datadir}/go/%{go_api}
%dir %{_sysconfdir}/gdbinit.d/
%config %{_sysconfdir}/gdbinit.d/go.gdb
%ghost %{_sysconfdir}/alternatives/go
%ghost %{_sysconfdir}/alternatives/gofmt
%ghost %{_sysconfdir}/alternatives/go.gdb
%dir %{_docdir}/go
%dir %{_docdir}/go/%{go_api}
%doc %{_docdir}/go/%{go_api}/AUTHORS
%doc %{_docdir}/go/%{go_api}/CONTRIBUTORS
%doc %{_docdir}/go/%{go_api}/CONTRIBUTING.md
%doc %{_docdir}/go/%{go_api}/LICENSE
%doc %{_docdir}/go/%{go_api}/PATENTS
%doc %{_docdir}/go/%{go_api}/README.md
%doc %{_docdir}/go/%{go_api}/README.SUSE

# We don't include TSAN in the main Go package.
%ifarch %{tsan_arch}
%exclude %{_datadir}/go/%{go_api}/src/runtime/race/race_linux_%{go_arch}.syso
%endif

%files doc
%defattr(-,root,root,-)
%doc %{_docdir}/go/%{go_api}/codewalk
%doc %{_docdir}/go/%{go_api}/articles
%doc %{_docdir}/go/%{go_api}/progs
%doc %{_docdir}/go/%{go_api}/play
%doc %{_docdir}/go/%{go_api}/gopher
%doc %{_docdir}/go/%{go_api}/*.html
%doc %{_docdir}/go/%{go_api}/*.css
%doc %{_docdir}/go/%{go_api}/*.png

%ifarch %{tsan_arch}
%files race
%defattr(-,root,root,-)
%{_datadir}/go/%{go_api}/src/runtime/race/race_linux_%{go_arch}.syso
%endif

%changelog
* Thu Apr  9 2020 Jeff Kowalczyk <jkowalczyk@suse.com>
- go1.14.2 (released 2020-04-08) includes fixes to cgo, the go command,
  the runtime, os/exec, and testing packages.
  Refs boo#1164903 go1.14 release tracking
  * go#38156 doc: BuildNameToCertificate deprecated in go 1.14 not mentioned in the release notes
  * go#38118 runtime/pprof: lostProfileEvent stack breaks gentraceback guarantee
  * go#38083 cmd/go/internal/test: data race in (*runCache).builderRunTest
  * go#38072 runtime: timer self-deadlock due to preemption point
  * go#38051 runtime: loops forever on sched_yield sometimes(timer related)
  * go#38005 runtime: "pipe failed with -89" at program startup(mipsle only), timer related netpoll init.
  * go#37970 runtime/pprof: panic: runtime error: index out of range [-1]
  * go#37968 runtime: fatal error: found bad pointer in Go heap (incorrect use of unsafe or cgo?)
  * go#37959 testing: data race between parallel panicking and normal subtest
  * go#37931 cmd/go: explain automatic vendoring in 'go help modules'
  * go#37928 runtime: GC pacing exhibits strange behavior with a low GOGC
  * go#37800 cmd/go: 'Access is denied' when renaming module cache directory
  * go#37699 PowerRegisterSuspendResumeNotification error on Azure App Services with go 1.13.7
  * go#37622 cmd/cgo: fails to generate certain types with Go 1.14
  * go#37480 runtime: "fatal error: unexpected signal" 0xC0000005 on Windows for a small program with a large allocation
  * go#37471 os/exec: environForSysProcAttr is never called as sysattr.Env is never nil
* Fri Mar 20 2020 Jeff Kowalczyk <jkowalczyk@suse.com>
- go1.14.1 (released 2020-03-19) includes fixes to the go command,
  tools, and the runtime.
  Refs boo#1164903 go1.14 release tracking
  * go#37905 cmd/compile: -d=checkptr should not reject unaligned pointers to non-pointer data
  * go#37833 runtime: sometimes 100%% CPU spin during init phase in Go 1.14 with preemptive scheduler
  * go#37822 cmd/go: module's "go" version should be included in cache key
  * go#37807 runtime: mlock of signal stack failed: 12
  * go#37782 runtime: crash on 1.14 with unexpected return pc, fatal error: unknown caller pc
  * go#37721 reflect: MakeMap() and not native map type wrong behavior
  * go#37671 cmd/go: tests that panic or exit are marked as passing when -json flag is used
  * go#37667 runtime: asyncPreempt should not try to save floating-point context for softfloat MIPS targets
  * go#37630 doc: missing documentation of quoting the URL of url.Errors in go1.14 release notes
  * go#37613 runtime: Go 1.14.rc1 3-5%% performance regression from 1.13 during protobuf marshalling
  * go#37494 time: racy Timer access should either work or throw, not panic
  * go#37478 SIGILL: illegal instruction on any go tool under macOS
  * go#37447 runtime/pprof: inline frames may not use combined location
  * go#37343 cmd/trace: requires HTML imports, which doesn't work on any major browser anymore
* Tue Mar  3 2020 Jeff Kowalczyk <jkowalczyk@suse.com>
- Packaging sync accumulated changes from go1.12
  Refs boo#1164903 go1.14 release tracking
- Use gcc9 by default by updating define gcc_go_version 9 (was 8)
  * drop unneeded patch gcc8-go.patch
- Fix broken go_api evaluation (1.12 < 1.5, when evaluated as floats),
  let RPM evaluate the expression, drop no longer required bc.
- Own the gdbinit.d directory, avoid the build dependency on gdb.
- Add %%ifarch %%arm aarch64 BuildRequires: binutils-gold to fix
  /usr/lib64/go/{version}/pkg/tool/linux_arm64/link: running gcc failed: exit status 1
  collect2: fatal error: cannot find 'ld'-
* Tue Feb 25 2020 Jeff Kowalczyk <jkowalczyk@suse.com>
- go1.14 (released 2020-02-25) is a major release of Go.
  go1.14.x minor releases will be provided through February 2021.
  https://github.com/golang/go/wiki/Go-Release-Cycle
  Most changes are in the implementation of the toolchain, runtime,
  and libraries. As always, the release maintains the Go 1 promise
  of compatibility. We expect almost all Go programs to continue to
  compile and run as before.
  Refs boo#1164903 go1.14 release tracking
  * See release notes https://golang.org/doc/go1.14. Excerpts
    relevant to OBS environment and for SUSE/openSUSE follow:
  * Module support in the go command is now ready for production
    use, and we encourage all users to migrate to Go modules for
    dependency management.
  * RISC-V experimental support for 64-bit RISC-V on Linux
    (GOOS=linux, GOARCH=riscv64). Be aware that performance,
    assembly syntax stability, and possibly correctness are a work
    in progress.
  * When the main module contains a top-level vendor directory and
    its go.mod file specifies go 1.14 or higher, the go command now
    defaults to -mod=vendor for operations that accept that flag. A
    new value for that flag, -mod=mod, causes the go command to
    instead load modules from the module cache (as when no vendor
    directory is present).
  * When -mod=vendor is set (explicitly or by default), the go
    command now verifies that the main module's vendor/modules.txt
    file is consistent with its go.mod file.
  * go list -m no longer silently omits transitive dependencies
    that do not provide packages in the vendor directory. It now
    fails explicitly if -mod=vendor is set and information is
    requested for a module not mentioned in vendor/modules.txt.
  * The go get command no longer accepts the -mod flag. Previously,
    the flag's setting either was ignored or caused the build to
    fail.
  * mod=readonly is now set by default when the go.mod file is
    read-only and no top-level vendor directory is present.
  * modcacherw is a new flag that instructs the go command to leave
    newly-created directories in the module cache at their default
    permissions rather than making them read-only. The use of this
    flag makes it more likely that tests or other tools will
    accidentally add files not included in the module's verified
    checksum. However, it allows the use of rm -rf (instead of go
    clean -modcache) to remove the module cache.
  * modfile=file is a new flag that instructs the go command to
    read (and possibly write) an alternate go.mod file instead of
    the one in the module root directory. A file named go.mod must
    still be present in order to determine the module root
    directory, but it is not accessed. When -modfile is specified,
    an alternate go.sum file is also used: its path is derived from
    the -modfile flag by trimming the .mod extension and appending
    .sum.
* Thu Feb  6 2020 Jeff Kowalczyk <jkowalczyk@suse.com>
- go1.14rc1 (released 2020/02/05) is a release candidate version of
  Go 1.14. It is cut from release-branch.go1.14 at the revision
  tagged go1.14rc1. go1.14rc1 is packaged before stable release of
  go1.14 to provide a preview of new default behavior for go
  modules. This early access is primarily intended to test use in
  offline OBS environment use with upstream go proxy infrastructure.
  * packaging: drop patch gcc9-rsp-clobber.patch now merged in go1.14
  * packaging: update version of LLVM compiler-rt
  * packaging: update _service definitions
  * packaging: update %%doc entries rm devel/ add modules.md
  * doc: rename HTML element IDs to avoid duplicates
  * net: don't check LookupHost error in TestLookupNullByte
  * runtime: don't treat SIGURG as a bad signal
  * internal/bytealg: fix riscv64 offset names
  * doc: remove paragraph break for upgrading to modules
  * syscall: Revert "release a js.Func object in fsCall"
  * doc/go1.14: note that all changes to the standard library are minor
  * doc/go1.14: fix broken links
  * doc/go1.14: remove TODO about Solaris port
